@@ -1,5 +1,5 @@
 # This file is part of RStan
-# Copyright (C) 2012, 2013, 2014, 2015, 2016 Jiqiang Guo and Benjamin Goodrich
+# Copyright (C) 2012, 2013, 2014, 2015, 2016 Trustees of Columbia University
 #
 # RStan is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -57,8 +57,16 @@ expose_stan_functions <- function(stanmodel) {
   lines <- gsub("vector<Eigen::Matrix<.*,Eigen::Dynamic> >", "vector<matrix_d>", lines)
   lines <- gsub("Eigen::Matrix<.*,Eigen::Dynamic>", "matrix_d", lines)
   
-  # kill foo_log<false> functions because of templating
-  templated <- grep("_log<false>", lines, fixed = TRUE)
+  # kill foo_lpdf<false> functions because of templating
+  templated <- grep("_lp[dm]f<false>", lines)
+  if(length(templated) > 0) for(i in rev(templated)) {
+    end <- i + 1L
+    while(!grepl("^}$", lines[end])) end <- end + 1L
+    start <- i - 1L
+    while(!grepl("^template", lines[start])) start <- start - 1L
+    lines <- lines[-c(start:end)]
+  }
+  templated <- grep("_log<false>", lines)
   if(length(templated) > 0) for(i in rev(templated)) {
     end <- i + 1L
     while(!grepl("^}$", lines[end])) end <- end + 1L
@@ -160,6 +168,14 @@ expose_stan_functions <- function(stanmodel) {
                 "\\1);", lines)
   lines <- gsub("([[:space:]]+return .*_rng)\\(base_rng__\\);",
                 "\\1();", lines)
+  lines <- gsub("_rng\\(base_rng__\\)", "_rng\\(seed, base_rng__\\)", lines)
+  RNGs <- grep("_rng\\(.*base_rng__", lines)
+  if (length(RNGs)) {
+    known_RNGs <- lookup("_rng$")[,1]
+    for (i in RNGs) if (!any(sapply(known_RNGs, FUN = grepl, x = lines[i])))
+      lines[i] <- gsub("base_rng__", "seed", lines[i], fixed = TRUE)
+  }
+  
   
   # remove line numbering things
   lines <- grep("current_statement_begin__", lines, 
@@ -176,6 +192,7 @@ expose_stan_functions <- function(stanmodel) {
   lines <- gsub(", T_lp_accum__& lp_accum__", "", lines, fixed = TRUE)
   lines <- gsub(", lp_accum__", "", lines, fixed = TRUE)
   lines <- gsub("get_lp(lp__)", "get_lp(lp__, lp_accum__)", lines, fixed = TRUE)
+
   
   # make propto__ false to not skip anything that is double
   lines <- gsub("const static bool propto__ = true;",
