@@ -73,7 +73,7 @@ rstanplugin <- function() {
   if (.Platform$OS.type == "windows")
     StanHeaders_pkg_libs <- system.file("libs", .Platform$r_arch, package = "StanHeaders")
   else StanHeaders_pkg_libs <- system.file("lib", package = "StanHeaders")
-  
+
   # In case  we have space (typical on windows though not necessarily)
   # in the file path of Rcpp's library. 
   
@@ -82,10 +82,11 @@ rstanplugin <- function() {
   if (grepl('[^\\\\]\\s', rcpp_pkg_libs, perl = TRUE))
     rcpp_pkg_libs <- gsub(rcpp_pkg_path, rcpp_pkg_path2, rcpp_pkg_libs, fixed = TRUE) 
 
+  
   list(includes = '',
        body = function(x) x,
        env = list(PKG_LIBS = paste(rcpp_pkg_libs,  
-                                   paste0("-L", StanHeaders_pkg_libs),
+                                   paste0("-L", shQuote(StanHeaders_pkg_libs)),
                                    "-lStanHeaders"),
                   PKG_CPPFLAGS = paste(Rcpp_plugin$env$PKG_CPPFLAGS,
                                         PKG_CPPFLAGS_env_fun(), collapse = " ")))
